from grammar import Rule, RuleProb, Grammar

class Container(dict):
	"""
	BaseClass for different containers: chart, count, inside, viterbi, etc.
	A Container is realized as a dict with:
		* key: container(s, A, s+1)
		* value: may vary in different container types (subclasses)
	"""
	def __init__(self, set, gram):
		self.initialize(set, gram)

	def initialize(self, set, gram):
		"""
		set initial values. Values are computed by init_func for each container.
		"""
		for s in range(len(set)):
			for rule in gram.getLexRules():
				if set[s] == rule.getRight():
					self[(s, rule.getLeft(), s+1)] = self.init_func(rule)
	
	def init_func(self, rule):
		"""
		should be redefined in a subclass
		"""
		pass



class Chart(Container):
	init_func = lambda a,b: True

class Count(Container):
	init_func = lambda a,b: 1

class Inside(Container):
	pass

class Viterbi(Container):
	pass


class ParserBase:
	def __init__(self, set, gram, C):
		"""
		Parser BaseClass takes sentence (i.e. a list of words), 
		a grammar and container type; then _parse method is called,
		for each parsing algorithm the same. They only differ in
		their own (subclassed) parse_func methods.
		"""
		self.gram = gram
		self._parse(set, C)

	def _parse(self, set, C):
		for l in range(2, len(set)+1):
			for s in range(len(set)-l+1):
				for t in range(1, l):
					for rule in self.gram.getRules():
						self.parse_func(l, s, t, C, rule)
		return C
	
	def parse_func(self, l, s, t, C, rule):
		"""
		should be redefined in a subclass
		"""
		pass


class CKYParser(ParserBase):
	def parse_func(self, l, s, t, chart, rule):
		#Hier euren Code einfuegen
		"""entsprechend den Zeilen 9/10 aus Figur 2.12 in 
		"EM-basierte maschinelle Lernverfahren fuer natuerliche Sprachen"
		(siehe Kurseite)"""
		pass

class CountParser(ParserBase):
	def parse_func(self, l, s, t, count, rule):
		#Hier euren Code einfuegen
		"""entsprechend den Zeilen 9/10 aus Figur 2.13"""
		pass


if __name__=="__main__":
	g = Grammar("Grammar.txt")
	set = "they study fish in cans".split()
	#CKY-Test-Run
	#contains NOW just the initial chart
	C = Chart(set, g)
	parser = CKYParser(set, g, C)
	for c in C:
		if C.get(c):
			print c, "=", C[c]
	
	print
	#Count-Test-Run	
	#contains NOW just the initial count 
	C = Count(set, g)
	parser = CountParser(set, g, C)
	for c in C:
		if C.get(c):
			print c, "=", C[c]
	#TODO: sich eine huebsche Visualisierung ueberlegen fuer die Chart, etc.
